/*

   ISL2SPLTask.java

   This task is a simple one-shot intended to convert all systems on an
   ISL network to the corresponding SPL network.
   
   Created: 25 September 2002
   Release: $Name:  $
   Version: $Revision: 1.11 $
   Last Mod Date: $Date: 2002/02/01 06:34:28 $
   Module By: Jonathan Abbey, jonabbey@arlut.utexas.edu

   -----------------------------------------------------------------------
	    
   Ganymede Directory Management System
 
   Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002
   The University of Texas at Austin.

   Contact information

   Author Email: ganymede_author@arlut.utexas.edu
   Email mailing list: ganymede@arlut.utexas.edu

   US Mail:

   Computer Science Division
   Applied Research Laboratories
   The University of Texas at Austin
   PO Box 8029, Austin TX 78713-8029

   Telephone: (512) 835-3200

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

package arlut.csd.ganymede.custom;

import arlut.csd.ganymede.*;

import java.util.*;
import java.text.*;
import java.io.*;
import java.rmi.*;

/*------------------------------------------------------------------------------
                                                                           class
                                                               PasswordAgingTask

------------------------------------------------------------------------------*/

/**
 *
 * This task is a simple one-shot intended to convert all systems on an
 * ISL network to the corresponding SPL network.
 *
 *
 * @author Jonathan Abbey jonabbey@arlut.utexas.edu
 */

public class ISL2SPLTask implements Runnable {

  static final boolean debug = true;

  /* -- */

  GanymedeSession mySession = null;
  Thread currentThread = null;

  String sourceNets[] = {"ISL - Network Equipment",
			 "ISL - No Inet Access",
			 "ISL - Outgoing Inet Access Only",
			 "ISL - Static NAT"};

  String targetNets[] = {"SPL - Network Equipment",
			 "SPL - No Inet Access",
			 "SPL - Outgoing Inet Access Only",
			 "SPL - Static NAT"};
  /**
   *
   * Just Do It (tm)
   *
   * @see java.lang.Runnable
   *
   */

  public void run()
  {
    boolean transactionOpen = false;

    /* -- */

    currentThread = java.lang.Thread.currentThread();

    Ganymede.debug("ISL2SPL Conversion Task: Starting");

    String error = GanymedeServer.checkEnabled();
	
    if (error != null)
      {
	Ganymede.debug("Deferring ISL2SPL Conversion task - semaphore disabled: " + error);
	return;
      }

    try
      {
	try
	  {
	    mySession = new GanymedeSession("ISL2SPLTask");
	  }
	catch (RemoteException ex)
	  {
	    Ganymede.debug("ISL2SPL Task: Couldn't establish session");
	    return;
	  }

	// we want interactive handholding

	mySession.enableWizards(true);

	// and we want forced required fields oversight..

	mySession.enableOversight(true);
	
	ReturnVal retVal = mySession.openTransaction("ISL2SPL conversion task");

	if (retVal != null && !retVal.didSucceed())
	  {
	    Ganymede.debug("ISL2SPL Task: Couldn't open transaction");
	    return;
	  }

	transactionOpen = true;
	
	// do the stuff

	if (!convertNetworks())
	  {
	    Ganymede.debug("ConvertNetworks bailed");

	    mySession.abortTransaction();
	    return;
	  }

	retVal = mySession.commitTransaction();

	if (retVal != null && !retVal.didSucceed())
	  {
	    // if doNormalProcessing is true, the
	    // transaction was not cleared, but was
	    // left open for a re-try.  Abort it.

	    if (retVal.doNormalProcessing)
	      {
		Ganymede.debug("ISL2SPL Task: couldn't fully commit, trying to abort.");

		mySession.abortTransaction();
	      }

	    Ganymede.debug("ISL2SPL Task: Couldn't successfully commit transaction");
	  }
	else
	  {
	    Ganymede.debug("ISL2SPL Task: Transaction committed");
	  }

	transactionOpen = false;
      }
    catch (InterruptedException ex)
      {
      }
    catch (Throwable ex)
      {
	Ganymede.debug("Caught " + ex.getMessage());
      }
    finally
      {
	if (transactionOpen)
	  {
	    Ganymede.debug("ISL2SPL Task: Forced to terminate early, aborting transaction");
	  }

	mySession.logout();
      }
  }

  private boolean convertNetworks() throws InterruptedException
  {
    Query q;
    Vector results;
    Result result;
    Invid invid;
    Enumeration enum;

    for (int i = 0; i < sourceNets.length; i++)
      {
	Ganymede.debug("Bulk converting " + sourceNets[i] + " to " + targetNets[i]);

	Invid sourceNet = mySession.findLabeledObject(sourceNets[i], (short) 267);
	Invid targetNet = mySession.findLabeledObject(targetNets[i], (short) 267);

	if (sourceNet == null)
	  {
	    Ganymede.debug("Couldn't find " + sourceNets[i]);
	    return false;
	  }

	if (targetNet == null)
	  {
	    Ganymede.debug("Couldn't find " + targetNets[i]);
	    return false;
	  }

	QueryDataNode matchNode = new QueryDataNode(interfaceSchema.IPNET,
						    QueryDataNode.EQUALS,
						    sourceNet);

	q = new Query((short) 265, matchNode, false);

	results = mySession.internalQuery(q);

	enum = results.elements();

	while (enum.hasMoreElements())
	  {
	    if (currentThread.isInterrupted())
	      {
		throw new InterruptedException("scheduler ordering shutdown");
	      }

	    result = (Result) enum.nextElement();

	    invid = result.getInvid();

	    ReturnVal retVal = mySession.edit_db_object(invid);

	    if (!retVal.didSucceed())
	      {
		return false;
	      }

	    DBEditObject interfaceObject = (DBEditObject) retVal.getObject();

	    if (interfaceObject == null)
	      {
		return false;
	      }

	    String interfaceLabel = interfaceObject.toString();

	    Invid sysInvid = (Invid) interfaceObject.getFieldValueLocal(SchemaConstants.ContainerField);

	    DBObject sysObj = mySession.getSession().viewDBObject(sysInvid);

	    Ganymede.debug("Converting interface in " + sysObj.toString());

	    // now we just need to flip the network over

	    IPDBField addressField = (IPDBField) interfaceObject.getField(interfaceSchema.ADDRESS);

	    //	    String oldAddress = addressField.getValueString();

	    InvidDBField networkField = (InvidDBField) interfaceObject.getField(interfaceSchema.IPNET);

	    retVal = networkField.setValue(targetNet);

	    if (retVal != null && !retVal.didSucceed())
	      {
		return false;
	      }

	    String newAddress = addressField.getValueString();

	    Ganymede.debug("Converted " + interfaceLabel + " to " + newAddress); 
	  }
      }

    return true;		// yikes!
  }
}
